{% extends "./browse.html" %}

{% comment %}
Copyright (C) 2017-2018  The Software Heritage developers
See the AUTHORS file at the top-level directory of this distribution
License: GNU Affero General Public License version 3, or any later version
See top-level LICENSE file for more information
{% endcomment %}

{% load swh_templatetags %}

{% block navbar-content %}
{% if not snapshot_context %}
  <h4>
    Browse person
  </h4>
{% else %}
  {{ block.super }}
{% endif %}
{% endblock %}


{% block swh-browse-content %}
<table class="table mt-3">
  <tbody>
    {% for key, val in swh_object_metadata.items|dictsort:"0.lower" %}
      <tr>
        <th class="swh-metadata-table-row swh-metadata-table-key">{{ key }}</th>
        <td class="swh-metadata-table-row swh-metadata-table-value">
          <pre>{{ val | safe | urlize_links_and_mails | safe }}</pre>
        </td>
      </tr>
    {% endfor %}
  </tbody>
</table>
{% endblock %}