.swh-vault-table{border-bottom:none!important;margin-bottom:0!important}.swh-vault-table td{vertical-align:middle!important;border-top:1px solid #ddd!important}
/*# sourceMappingURL=vault.d3301724e72732613d37.css.map */