.swh-showdown a{border:none}.swh-showdown table{border-collapse:collapse}.swh-showdown table,.swh-showdown table td,.swh-showdown table th{padding:6px 13px;border:1px solid #dfe2e5}.swh-showdown table tr:nth-child(2n){background-color:#f2f2f2}
/*# sourceMappingURL=showdown.1379127bb1663f9405fb.css.map */